==> weather-2.01.tar.bz2: weather/weather_net.cpp <==
/***************************************************************************
 *   Weather module for Kadu communicator                                  *
 *   2004 Micha Maek (bysk)                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "weather_net.h"

/**
	SearchIDLocation
**/

SearchIDLocation::SearchIDLocation(ConfigFile* cf, const QStringList& wCfFiles)
:
	parser(strPage, cf),
	weatherConfigFiles(wCfFiles)
{
	timer_timeout = new QTimer(this);

	connect(timer_timeout, SIGNAL(timeout()), this, SLOT(connectionTimeout()));
	connect(&httpclient, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	connect(&httpclient, SIGNAL(error()), this, SLOT(downloadingError()));
	connect(&httpclient, SIGNAL(redirected(QString)), this, SLOT(downloadingRedirected(QString)));
}

SearchIDLocation::~SearchIDLocation()
{
	disconnect(&httpclient, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	disconnect(&httpclient, SIGNAL(error()), this, SLOT(downloadingError()));
	disconnect(&httpclient, SIGNAL(redirected(QString)), this, SLOT(downloadingRedirected(QString)));
	disconnect(timer_timeout, SIGNAL(timeout()), this, SLOT(connectionTimeout()));
}

bool SearchIDLocation::findID(QString loc_name, int config_pos)
{
	locName = loc_name;
	cfPos = config_pos;
	
	strPage = "";	
	QString wcfName = weatherConfigFiles[config_pos];
	ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+wcfName));
		
	QString encoding = wConfig.readEntry("Default","Encoding");
	decoder = QTextCodec::codecForName(encoding);
	
	httpclient.setHost(wConfig.readEntry("Name Search","Search host"));
	
	if(loc_name.isEmpty())
	{
		if(wConfig.readBoolEntry("Name Search","AllLocations"))
			url = wConfig.readEntry("Name Search","AllLocationsPath");
		else
			return false;
	}
	else
	{
		encodeUrl(loc_name, encoding);
		url.sprintf(wConfig.readEntry("Name Search","Search path"),(const char*)loc_name);
	}
	
	timer_timeout->start(CONNECTION_TIMEOUT, false);
	timeoutCount = CONNECTION_COUNT;
	httpclient.get(url);
	return true;
}

void SearchIDLocation::downloadingFinished()
{
	timer_timeout->stop();

	const QByteArray& data = httpclient.data();
	strPage = decoder->toUnicode( data.data(), data.count());
	
	QString wcfName = weatherConfigFiles[cfPos];
	ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+wcfName));
	
	names.clear();
	codes.clear();
	parser.getSearch(names, codes, &wConfig);
	
	for(uint i = 0; i < names.count(); i++)
	{
		// Jeli strona zawiera wszystkie miasta,
		// trzeba dokadniej przefiltrowa
		if(wConfig.readBoolEntry("Name Search","OnePage") &&
			names[i].find(locName, 0, false) == -1)
		{
			names.remove(names.at(i));
			codes.remove(codes.at(i));
			i--;
			continue;	
		}
	}
	
	emit finished();
}

void SearchIDLocation::downloadingRedirected(QString link)
{
	timer_timeout->stop();

	httpclient.setHost("");
	
	QString wcfName = weatherConfigFiles[cfPos];
	ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+wcfName));

	names.clear();
	codes.clear();
	names.push_back(locName);
	codes.push_back(parser.getFastSearch(link, wConfig));
	
	//emit redirected(link);
	emit finished();
}

void SearchIDLocation::downloadingError()
{
	timer_timeout->stop();
	emit error();
}

void SearchIDLocation::connectionTimeout()
{
	if(--timeoutCount <= 0)
	{
		httpclient.setHost("");
		downloadingError();
	}
	else
	{
		httpclient.setHost("");
		httpclient.setHost(url);
	}
}

void SearchIDLocation::encodeUrl(QString& str, const QString& enc)
{
	QUrl::encode(str);
	
	if(enc == "ISO8859-2")
	{
		//QString rep[18][2]={{"","%B1"},{"","%E6"},{"","%EA"},{"","%B3"},{"%","%F1"},{"","%F3"},{"","%B6"},{"","%BF"},{"","%BC"},
		//					{"","%A1"},{"","%C6"},{"","%CA"},{"","%A3"},{"","%D1"},{"","%D3"},{"","%A6"},{"","%AF"},{"","%AC"}};
	
		QString rep[18][2]={{"%C4%85","%B1"},{"%C4%87","%E6"},{"%C4%99","%EA"},{"%C5%84","%F1"},{"%C5%82","%B3"},{"%C3%B3","%F3"},{"%C5%9B","%B6"},{"%C5%BC","%BF"},{"%C5%BA","%BC"},
							{"%C4%84","%A1"},{"%C4%86","%C6"},{"%C4%98","%CA"},{"%C5%83","%D1"},{"%C5%81","%A3"},{"%C3%93","%D3"},{"%C5%9A","%A6"},{"%C5%BB","%AF"},{"%C5%B9","%AC"}};
	
		for(int i=0; i<18; i++)
		{
			str.replace(rep[i][0],rep[i][1]);
		}
	}
}

/**
	GetForecast
**/
GetForecast::GetForecast(ConfigFile* cf, Forecast& f)
:
	forecast(f),
	parser(strPage, cf),
	//moduleConfig(cf),
	wConfig(NULL)
{
	timer_timeout = new QTimer(this);

	connect(&httpclient, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	connect(&httpclient, SIGNAL(error()), this, SLOT(downloadingError()));
	connect(timer_timeout, SIGNAL(timeout()), this, SLOT(connectionTimeout()));
}

GetForecast::~GetForecast()
{
	disconnect(timer_timeout, SIGNAL(timeout()), this, SLOT(connectionTimeout()));
	disconnect(&httpclient, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	disconnect(&httpclient, SIGNAL(error()), this, SLOT(downloadingError()));
	delete wConfig;
}

void GetForecast::downloadForecast(const QString& configFile, const QString& locID)
{
	forecast.Days.clear();
	forecast.LocationName = "";
	forecast.config = "";
	forecast.loadTime = QTime();

	if(wConfig != NULL)
		delete wConfig;

	//forecast.config = moduleConfig->readEntry("Configs",uin);
	forecast.LocationID = locID;
	forecast.config = configFile;
	QString wconf;
	if(forecast.config.isNull())
		return;
		//wconf = moduleConfig->readEntry("Weather","ServerConfigFile");
	//else
	//{
		wconf = dataPath("kadu/modules/data/weather/" + forecast.config);
		iconPath = dataPath("kadu/modules/data/weather/icons/");
	//}
	wConfig = new ConfigFile(wconf);
	
	forecast.serverName = wConfig->readEntry("Header","Name");
	
	decoder = QTextCodec::codecForName(wConfig->readEntry("Default","Encoding"));
	
	httpclient.setHost(wConfig->readEntry("Default","Default host"));
	//url.sprintf(wConfig->readEntry("Default","Default path"),
	//				(const char*)moduleConfig->readEntry("Locations",uin));
	url.sprintf(wConfig->readEntry("Default","Default path"),
					(const char*)locID);
	
	timer_timeout->start(CONNECTION_TIMEOUT, false);
	timeoutCount = CONNECTION_COUNT;
	
	httpclient.get(url);
}

void GetForecast::downloadingFinished()
{
	timer_timeout->stop();
	
	const QByteArray& data = httpclient.data();
	strPage = decoder->toUnicode( data.data(), data.count());
	
	parser.getData(wConfig, forecast, iconPath);
	forecast.loadTime.start();
	
	emit finished();
}

void GetForecast::downloadingError()
{
	timer_timeout->stop();
	emit error();
}

void GetForecast::connectionTimeout()
{
	if(--timeoutCount <= 0)
	{
		httpclient.setHost("");
		downloadingError();
	}
	else
	{
		httpclient.setHost("");
		httpclient.setHost(url);
	}
	
	//MessageBox::msg("connectionTimeout()");
}

==> weather-2.02.tar.bz2: weather/weather_net.cpp <==
/***************************************************************************
 *   Weather module for Kadu communicator                                  *
 *   2004 Micha Maek (bysk)                                             *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "weather_net.h"

/**
	SearchIDLocation
**/

SearchIDLocation::SearchIDLocation(ConfigFile* cf, const QStringList& wCfFiles)
:
	parser(strPage, cf),
	weatherConfigFiles(wCfFiles)
{
	timer_timeout = new QTimer(this);

	connect(timer_timeout, SIGNAL(timeout()), this, SLOT(connectionTimeout()));
	connect(&httpclient, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	connect(&httpclient, SIGNAL(error()), this, SLOT(downloadingError()));
	connect(&httpclient, SIGNAL(redirected(QString)), this, SLOT(downloadingRedirected(QString)));
}

SearchIDLocation::~SearchIDLocation()
{
	disconnect(&httpclient, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	disconnect(&httpclient, SIGNAL(error()), this, SLOT(downloadingError()));
	disconnect(&httpclient, SIGNAL(redirected(QString)), this, SLOT(downloadingRedirected(QString)));
	disconnect(timer_timeout, SIGNAL(timeout()), this, SLOT(connectionTimeout()));
}

bool SearchIDLocation::findID(QString loc_name, int config_pos)
{
	kdebugf();
	
	locName = loc_name;
	cfPos = config_pos;
	
	strPage = "";	
	QString wcfName = weatherConfigFiles[config_pos];
	ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+wcfName));
		
	QString encoding = wConfig.readEntry("Default","Encoding");
	decoder = QTextCodec::codecForName(encoding);
	
	host = wConfig.readEntry("Name Search","Search host");
	httpclient.setHost(host);
	
	if(loc_name.isEmpty())
	{
		if(wConfig.readBoolEntry("Name Search","AllLocations"))
			url = wConfig.readEntry("Name Search","AllLocationsPath");
		else
			return false;
	}
	else
	{
		encodeUrl(loc_name, encoding);
		url.sprintf(wConfig.readEntry("Name Search","Search path"),(const char*)loc_name);
	}
	
	timer_timeout->start(WEATHER_CONNECTION_TIMEOUT, false);
	timeoutCount = WEATHER_CONNECTION_COUNT;
	httpclient.get(url);
	
	kdebugf2();
	return true;
}

void SearchIDLocation::downloadingFinished()
{
	kdebugf();
	
	timer_timeout->stop();

	const QByteArray& data = httpclient.data();
	strPage = decoder->toUnicode( data.data(), data.count());
	
	QString wcfName = weatherConfigFiles[cfPos];
	ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+wcfName));
	
	names.clear();
	codes.clear();
	parser.getSearch(names, codes, &wConfig);
	
	for(uint i = 0; i < names.count(); i++)
	{
		// Jeli strona zawiera wszystkie miasta,
		// trzeba dokadniej przefiltrowa
		if(wConfig.readBoolEntry("Name Search","OnePage") &&
			names[i].find(locName, 0, false) == -1)
		{
			names.remove(names.at(i));
			codes.remove(codes.at(i));
			i--;
			continue;	
		}
	}
	
	emit finished();
	
	kdebugf2();
}

void SearchIDLocation::downloadingRedirected(QString link)
{
	kdebugf();
	
	QString wcfName = weatherConfigFiles[cfPos];
	ConfigFile wConfig(dataPath("kadu/modules/data/weather/"+wcfName));
	QString id = parser.getFastSearch(link, wConfig);
	
	httpclient.setHost("");
	
	if(id.isEmpty())
	{
		splitUrl(link, host, url);
		httpclient.setHost(host);
		httpclient.get(url);
	}
	else
	{
		timer_timeout->stop();
		
		names.clear();
		codes.clear();
		names.push_back(locName);
		codes.push_back(id);
		
		//emit redirected(link);
		emit finished();
	}

	kdebugf2();
}

void SearchIDLocation::downloadingError()
{
	kdebugf();
	
	timer_timeout->stop();
	emit error(host + '/' + url);

	kdebugf2();
}

void SearchIDLocation::connectionTimeout()
{
	kdebugf();
	
	if(--timeoutCount <= 0)
	{
		httpclient.setHost("");
		downloadingError();
	}
	else
	{
		httpclient.setHost("");
		httpclient.setHost(host);
		httpclient.get(url);
	}

	kdebugf2();
}

void SearchIDLocation::encodeUrl(QString& str, const QString& enc)
{
	kdebugf();
	
	QUrl::encode(str);
	
	if(enc == "ISO8859-2")
	{
		//QString rep[18][2]={{"","%B1"},{"","%E6"},{"","%EA"},{"","%B3"},{"%","%F1"},{"","%F3"},{"","%B6"},{"","%BF"},{"","%BC"},
		//					{"","%A1"},{"","%C6"},{"","%CA"},{"","%A3"},{"","%D1"},{"","%D3"},{"","%A6"},{"","%AF"},{"","%AC"}};
	
		QString rep[18][2]={{"%C4%85","%B1"},{"%C4%87","%E6"},{"%C4%99","%EA"},{"%C5%84","%F1"},{"%C5%82","%B3"},{"%C3%B3","%F3"},{"%C5%9B","%B6"},{"%C5%BC","%BF"},{"%C5%BA","%BC"},
							{"%C4%84","%A1"},{"%C4%86","%C6"},{"%C4%98","%CA"},{"%C5%83","%D1"},{"%C5%81","%A3"},{"%C3%93","%D3"},{"%C5%9A","%A6"},{"%C5%BB","%AF"},{"%C5%B9","%AC"}};
	
		for(int i=0; i<18; i++)
		{
			str.replace(rep[i][0],rep[i][1]);
		}
	}

	kdebugf2();
}

/**
	GetForecast
**/
GetForecast::GetForecast(ConfigFile* cf, Forecast& f)
:
	forecast(f),
	parser(strPage, cf),
	//moduleConfig(cf),
	wConfig(NULL)
{
	timer_timeout = new QTimer(this);

	connect(&httpclient, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	connect(&httpclient, SIGNAL(error()), this, SLOT(downloadingError()));
	connect(timer_timeout, SIGNAL(timeout()), this, SLOT(connectionTimeout()));
}

GetForecast::~GetForecast()
{
	disconnect(timer_timeout, SIGNAL(timeout()), this, SLOT(connectionTimeout()));
	disconnect(&httpclient, SIGNAL(finished()), this, SLOT(downloadingFinished()));
	disconnect(&httpclient, SIGNAL(error()), this, SLOT(downloadingError()));
	delete wConfig;
}

void GetForecast::downloadForecast(const QString& configFile, const QString& locID)
{
	kdebugf();
	
	forecast.Days.clear();
	forecast.LocationName = "";
	forecast.config = "";
	forecast.loadTime = QTime();

	if(wConfig != NULL)
		delete wConfig;

	//forecast.config = moduleConfig->readEntry("Configs",uin);
	forecast.LocationID = locID;
	forecast.config = configFile;
	QString wconf;
	if(forecast.config.isNull())
		return;
		//wconf = moduleConfig->readEntry("Weather","ServerConfigFile");
	//else
	//{
		wconf = dataPath("kadu/modules/data/weather/" + forecast.config);
		iconPath = dataPath("kadu/modules/data/weather/icons/");
	//}
	wConfig = new ConfigFile(wconf);
	
	forecast.serverName = wConfig->readEntry("Header","Name");
	
	decoder = QTextCodec::codecForName(wConfig->readEntry("Default","Encoding"));
	
	host = wConfig->readEntry("Default","Default host");
	httpclient.setHost(host);
	//url.sprintf(wConfig->readEntry("Default","Default path"),
	//				(const char*)moduleConfig->readEntry("Locations",uin));
	url.sprintf(wConfig->readEntry("Default","Default path"),
					(const char*)locID);
	
	timer_timeout->start(WEATHER_CONNECTION_TIMEOUT, false);
	timeoutCount = WEATHER_CONNECTION_COUNT;
	
	httpclient.get(url);

	kdebugf2();
}

void GetForecast::downloadingFinished()
{
	kdebugf();
	
	timer_timeout->stop();
	
	const QByteArray& data = httpclient.data();
	strPage = decoder->toUnicode( data.data(), data.count());
	
	if(!parser.getData(wConfig, forecast, iconPath))
	{
		emit error(WEATHER_ERROR_PARSER, host + '/' + url);
		return;
	}
	
	forecast.loadTime.start();
	
	emit finished();

	kdebugf2();
}


void GetForecast::downloadingRedirected(QString link)
{
	kdebugf();
	
	splitUrl(link, host, url);
	httpclient.setHost(host);
	httpclient.get(url);

	kdebugf2();
}

void GetForecast::downloadingError()
{
	kdebugf();
	
	timer_timeout->stop();
	emit error(WEATHER_ERROR_CONNECTION, host + '/' + url);

	kdebugf2();
}

void GetForecast::connectionTimeout()
{
	kdebugf();
	
	timeoutCount = timeoutCount - 1;
	if(timeoutCount <= 0)
	{
		httpclient.setHost("");
		downloadingError();
	}
	else
	{
		httpclient.setHost("");
		httpclient.setHost(host);
		httpclient.get(url);
	}
	
	//MessageBox::msg("connectionTimeout()");
	kdebugf2();
}

void splitUrl(const QString& url, QString& host, QString& path)
{
	int endhost = url.find('/');
	host = url.left(endhost);
	path = url.right(url.length()-endhost);
}
